/* === This file is part of Calamares - <http://github.com/calamares> ===
 *
 *   Copyright 2018, Adriaan de Groot <groot@kde.org>
 *
 *   Calamares is free software: you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation, either version 3 of the License, or
 *   (at your option) any later version.
 *
 *   Calamares is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with Calamares. If not, see <http://www.gnu.org/licenses/>.
 */

#ifndef GEOIPFREEGEOIP_H
#define GEOIPFREEGEOIP_H

#include "GeoIP.h"

/** @brief GeoIP lookup via freegeoip.com
 *
 * This is the original implementation of GeoIP lookup,
 * using the FreeGeoIP service, or similar which returns
 * data in the same format.
 *
 * The data is assumed to be in JSON format with a time_zone attribute.
 */
struct FreeGeoIP : public GeoIP
{
    virtual RegionZonePair processReply( const QByteArray& );
} ;

#endif
